#!/usr/bin/env python3
"""
Memex - Personal Knowledge Assistant

Run with:
    python -m memex [options]

Or:
    python examples/memex [options]
"""

from .cli import main

if __name__ == "__main__":
    main()
